<?php

ignore_user_abort(false);

require dirname(__FILE__) . '/lib/common.php';

/**
 * path
 */
define('PATH_CONFIG', SetupCommon::fixPathSeparator(SCRIPT_DIR . 'config.server.php'));
define('PATH_LICENSE', SetupCommon::fixPathSeparator(SCRIPT_DIR . 'license.php'));
define('PATH_HTACCESS', SetupCommon::fixPathSeparator(SCRIPT_DIR . '.htaccess'));
define('PATH_ARCHIVES', SetupCommon::fixPathSeparator(SCRIPT_DIR . ARCHIVES_DIR));
define('PATH_THEMES', SetupCommon::fixPathSeparator(SCRIPT_DIR . THEMES_DIR));
define('REGEX_VALID_DOMAIN', '@^([a-zA-Z0-9]+[a-zA-Z0-9-]*(?<=[a-zA-Z0-9])\.)+[a-zA-Z]+[a-zA-Z0-9-]*(?<=[a-zA-Z0-9])\.?$|^(?:\d{1,3}\.){3}\d{1,3}$|^localhost$@');

if ( !defined('CACHE_DIR') ) {
    define('CACHE_DIR', 'cache/');
}
define('PATH_CACHE', SetupCommon::fixPathSeparator(SCRIPT_DIR . CACHE_DIR));



$own    = preg_replace('@^.*/(.*)$@si', '$1', SetupCommon::fixPathSeparator(dirname(__FILE__)));
if ( !defined('SETUP_DIR') ) define('SETUP_DIR', $own);

define('CONVERT4CATEGORY', true);
$baseUrl = (HTTPS ? 'https' : 'http').'://'.HTTP_HOST.(!!HTTP_PORT ? ':'.HTTP_PORT : '').'/';
define('HTTP_ROOT', rtrim($baseUrl.DIR_OFFSET, '/').'/');
$root = SetupCommon::fixPathSeparator(preg_replace('@'.SETUP_DIR.'$@', '', dirname(__FILE__)));
define('ROOT_DIR', $root);

class Setup
{
    var $Post;
    var $Get;

    function initialize()
    {
        try {
            $DB = DB::singleton(SetupCommon::dsn());

            $q  = "SET SESSION sql_mode=''";
            $DB->query($q, 'exec');
        } catch ( Exception $e ) {
            App::deleteExceptionStack();
        }

        $this->initSession();
        setcookie(SESSION_NAME, '', time() - 1, '/', $_SERVER['HTTP_HOST'], false, true);

        // initialize SEPARATOR
        if ( !defined('DIRECTORY_SEPARATOR') ) {
            if ( strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN' ) {
                define('DIRECTORY_SEPARATOR', '\\');
            } else {
                define('DIRECTORY_SEPARATOR', '/');
            }
        }

        // initialize EOL
        if ( !defined('PHP_EOL') ) {
            $env = strtoupper(substr(PHP_OS, 0, 3));
            switch ( $env ) {
                case 'WIN'  :
                    define('PHP_EOL', "\r\n");
                    break;
                case 'MAC'  :
                    define('PHP_EOL', "\r");
                    break;
                default     :
                    define('PHP_EOL', "\n");
                    break;
            }
        }

        // initialize POST & sanitizing
        $this->Post = new Field();
        if ( !empty($_POST) ) {
            array_walk($_POST, array('Common', 'fixEncodingAssoc'));
            foreach ( $_POST as $key => $val ) {
                if ( is_array($val) ) {
                    foreach ( $val as $k => $v ) {
                        $ary[]  = htmlentities($v, ENT_QUOTES, 'UTF-8');
                    }
                    $this->Post->set($key, $ary);
                } else {
                    $this->Post->set($key, htmlentities($val, ENT_QUOTES, 'UTF-8'));
                }
            }
        }

        // initialize POST & sanitizing
        $this->Get = new Field();
        if ( !empty($_GET) ) {
            array_walk($_GET, array('Common', 'fixEncodingAssoc'));
            foreach ( $_GET as $key => $val ) {
                if ( is_array($val) ) continue;
                $this->Get->set($key, htmlentities($val, ENT_QUOTES, 'UTF-8'));
            }
        }
    }

    function initSession()
    {
        session_name('acmsSetup');
        session_set_cookie_params(0);
        session_start();
        session_regenerate_id(true);
    }

    function checkAuth()
    {
        return !empty($_SESSION['auth']) ? true : false;
    }
}

class Setup_Detector extends Setup
{
    function __construct()
    {
        $Tpl = SetupCommon::setTpl(dirname(__FILE__).'/tpl/index.html');

        $this->initSession();
        $error = array();

        /**
         * check installed system
         */
        if ( !$this->IsInstalled() ) {
            die(header('Location: ./install.php'));
        }

        /**
         * try login
         */
        if ( !empty($_POST['id']) && !empty($_POST['pass']) && empty($_SESSION['auth']) ) {
            $id     = htmlentities($_POST['id'], ENT_QUOTES, 'UTF-8');
            $pass   = htmlentities($_POST['pass'], ENT_QUOTES, 'UTF-8');
            if ( $row = $this->loginAuth($id, $pass) ) {
                $_SESSION['auth'] = $row;
            } else {
                $error[] = 'login_auth';
            }
        }

        /**
         * detect root block
         */
        if ( !empty($_SESSION['auth']) ) {

            $root = 'admin';

            if ( !$this->detectUpdate($Tpl, $root) ) {
                $this->detectPlugin($Tpl, $root);
                $this->detectImport($Tpl, $root);
                $this->detectExport($Tpl, $root);
            }

            $this->detectFixDomain($Tpl, $root);
            $this->detectFixCommentAlign($Tpl, $root);
            $this->detectFixCategoryAlign($Tpl, $root);
            $this->detectFixBlogAlign($Tpl, $root);
            $this->detectFieldSearch($Tpl, $root);
            $this->detectFixFulltext($Tpl, $root);
            $this->detectFixSequence($Tpl, $root);
            $this->detectFixSummary($Tpl, $root);
            $this->detectFixAccess($Tpl, $root);
            $this->detectFixImage($Tpl, $root);
            $this->detectFixIndex($Tpl, $root);

            $this->detectExtAddNgram($Tpl, $root);
            $this->detectExtAddTags($Tpl, $root);

        } else {

            $root = 'curtain';

        }

        /**
         * output error log
         */
        foreach ($error as $key => $val) {
            if ( !!is_int($key) ) {
                $Tpl->add(array($val.':error', $root));
            } else {
                $Tpl->add(array($val.':error', $key, $root));
            }
        }
        $Tpl->add($root);
        print SetupCommon::getTpl($Tpl);
    }

    function detectUpdate(& $Tpl, $root)
    {
        $DB = DB::singleton(SetupCommon::dsn());
        $SQL = SQL::newSelect('sequence');
        $SQL->addSelect('sequence_system_version');
        $version = $DB->query($SQL->get(SetupCommon::dsn()), 'one');

        if ( version_compare($version, VERSION, '<') ) {
            $Tpl->add(array('update', $root));
            return true;
        } elseif ( version_compare($version, VERSION, '=') ) {
            $Tpl->add(array('current', $root));
            return false;
        } elseif ( version_compare($version, VERSION, '>') ) {
            header('Content-type: text/html; charset=UTF-8');
            die("<p>Setupプログラムがデータベースのバージョンよりも古い可能性があります。最新のsetupに差し替えてから実行してください。(Setup: ".VERSION.", Database: $version )</p>");
        } else {
            $Tpl->add(array('update', $root));
            return true;
        }
    }

    function detectPlugin(& $Tpl, $root)
    {
        if ( LICENSE_PERIOD == null && LICENSE_EXPIRE == null ) {
            $Tpl->add(array('plugin', $root));
        }
    }

    function detectImport(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('import', $root));
        }
    }

    function detectExport(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('export', $root));
        }
    }

    function detectFixDomain(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_domain', $root));
        }
    }

    function detectFixCommentAlign(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_comment-align', $root));
        }
    }

    function detectFixCategoryAlign(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_category-align', $root));
        }
    }

    function detectFixBlogAlign(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_blog-align', $root));
        }
    }

    function detectFieldSearch(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_field-search', $root));
        }
    }

    function detectFixFulltext(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_fulltext', $root));
        }
    }

    function detectFixSequence(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_sequence', $root));
        }
    }

    function detectFixSummary(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_summary', $root));
        }
    }

    function detectFixImage(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_image', $root));
        }
    }

    function detectFixAccess(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->Add(array('fix_access', $root));
        }
    }

    function detectExtAddNgram(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('extend_add-ngram', $root));
        }
    }

    function detectExtAddTags(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('extend_add-tags', $root));
        }
    }

    function detectFixIndex(& $Tpl, $root)
    {
        if ( 1 ) {
            $Tpl->add(array('fix_index'), $root);
        }
    }

    function loginAuth($id, $pass)
    {
        $DB = DB::singleton(SetupCommon::dsn());
        $SQL = SQL::newSelect('user');
        $SQL->addWhereOpr('user_auth', 'administrator');
        $W  = SQL::newWhere();
        $W->addWhereOpr('user_code', $id, '=', 'OR');
        $W->addWhereOpr('user_mail', $id, '=', 'OR');
        $SQL->addWhere($W);

        $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');
        $all = array_filter($all, function ($user) use ($pass) {
            return acmsUserPasswordVerify($pass, $user['user_pass'], getPasswordGeneration($user));
        });
        if (empty($all)) {
            return false;
        }
        $user = $all[0];
        return $user['user_id'];
    }

    public static function IsInstalled()
    {
        try {
            $dsn = dsn();
            $DB = DB::singleton($dsn);

            if ( empty($dsn['name']) ) {
                return false;
            }

            $SQL = SQL::newSelect('blog');
            $SQL->addWhereOpr('blog_id', 1);

            if ( $blog = $DB->query($SQL->get(dsn()), 'row') ) {
                return true;
            }

            App::checkException();
        } catch ( Exception $e) {
            App::deleteExceptionStack();
        }

        return false;
    }
}

class Surfacer extends Setup
{
    var $Error; // is_array
    var $Notice;// is_array
    var $Trans; // is_array
    var $Fetal; // is_bool
    var $_dump;

    /*
    abstract function buildSurface();
    abstract function validate();
    abstract function business();
    */

    public function __construct($tplPath)
    {
        $this->initialize();
        if ( !$this->checkAuth() ) {
            header('Content-type: text/html; charset=UTF-8');
            die("メンテナンス実行に対する権限がありません。管理者はメンテナンスメニューからログインし直してください。");
        }
        $Tpl  = SetupCommon::setTpl($tplPath);

        /**
         * if post data exists. when try to validation.
         */
        if ( !empty($_POST) ) {
            $this->validate();
        }

        /**
         * if current step is *result*. when try to run business logic.
         */
        if ( $this->Post->get('step') == 'result' ) {
            $this->business();
        }
        $root = 'step#'.$this->Post->get('step', 'apply');
        $this->buildSurface($Tpl, $root);
        $this->buildError($Tpl, $root);
        $Tpl->add($root, $this->Trans);

        print SetupCommon::getTpl($Tpl);
    }

    function detectStep()
    {
        $step = empty($this->Error) ? 'result' : 'error';
        $step = empty($this->Fetal) ? $step : 'fetal';
        $this->Post->set('step', $step);
    }

    function buildError(& $Tpl, $root)
    {
        if ( !empty($this->Error) ) {
            foreach ( $this->Error as $key => $val ) {
                if ( is_array($val) ) {
                    foreach ( $val as $k => $v ) {
                        $loop = array($key.':loop', $root);
                        $Tpl->add($loop, array($key => $k));
                    }
                }
                $error = array($key.':error', $root);
                $Tpl->add($error);
            }
            $this->Trans = $_POST;
            return true;
        } else {
            $this->Trans = $_POST;
            return false;
        }
    }

    function listBlog(& $Tpl, $root)
    {
        /**
         * list blog
         */
        $DB = DB::singleton(SetupCommon::dsn());
        $SQL = SQL::newSelect('blog');
        $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

        foreach ( $all as $row ) {
            $vars = array(
                        'bid'    => $row['blog_id'],
                        'name'   => $row['blog_name'],
                        'code'   => $row['blog_code'],
                        'parent' => $row['blog_parent'],
                        );
            if ( $row['blog_id'] == $this->Post->get('bid') ) $vars += array('checked' => ' checked="checked"');

            /**
             * list category
             */
            if ( CONVERT4CATEGORY === true ) {
                $SQL = SQL::newSelect('category');
                $SQL->addOrder('category_sort', 'ASC');
                $SQL->addWhereOpr('category_blog_id', $row['blog_id']);
                $_all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');
                foreach ( $_all as $_row ) {
                    $_vars = array(
                                'cid'     => $_row['category_id'],
                                'cname'   => $_row['category_name'],
                                'ccode'   => $_row['category_code'],
                                'cparent' => $_row['category_parent'],
                                );
                    if ( $_row['category_id'] == $this->Post->get('cid') ) $_vars += array('checked' => ' checked="checked"');
                    $Tpl->add(array('category:loop', 'blog:loop', $root), $_vars);
                }
            }

            $Tpl->add(array('blog:loop', $root), $vars);
        }
    }
}
